local _G = _G

local NUM_SPECS = 4

local MooUI_tblTrack = {}
-- This will contain all the forms of currently tracked auras

local MooUI_tblTrackPerSpec = {}
for i=0,NUM_SPECS do
    MooUI_tblTrackPerSpec[i] ={}
end
-- Self explanatory, I think

------------------------------------------------------------------

local _GetSpecialization = GetSpecialization
local function GetSpecialization ()
    return _GetSpecialization() or 0
end

------------------------------------------------------------------

local curFrame

local SHOW_ALWAYS       = 1
local SHOW_IN_COMBAT    = 2
local SHOW_WHEN_EXISTS  = 3
local SHOW_LOCK         = 10

local function ShowWhen (when)
    g_tblContextMenuTrack[2].menuList[curFrame.showWhen].checked = false
    
    curFrame.showWhen = when
    c_tblTrack[GetSpecialization()][curFrame.name].showWhen = when
    
    g_tblContextMenuTrack[2].menuList[when].checked = true
    
    if when==SHOW_ALWAYS then
        curFrame:Show()
    elseif when==SHOW_IN_COMBAT then
        if _G.UnitAffectingCombat('player') then
            curFrame:Show()
        else
            curFrame:Hide()
        end
    elseif when==SHOW_WHEN_EXISTS then
        local name = _G.GetAuraData(curFrame.target, curFrame.name:trim())
        if name then
            curFrame:Show ()
        else
            curFrame:Hide()
        end
    end
end

------------------------------------------------------------------

local tblContextMenuTrack = {
    { 
        text = "Options", 
        isTitle = true,
        notCheckable = true,
    },
    { 
        text = "Show...", 
        notCheckable = true,
        nested = true,
        hasArrow = true,
        menuList = {
            {
                text = "Always",
                notCheckable = false, 
                arg1 = SHOW_ALWAYS,
                func = function (self, arg1)
                    ShowWhen (arg1)
                end,
            },
            {
                text = "Only in combat",
                notCheckable = false, 
                arg1 = SHOW_IN_COMBAT,
                func = function (self, arg1)
                    ShowWhen (arg1)
                end,
            },
            {
                text = "Only when the aura exists",
                notCheckable = false, 
                arg1 = SHOW_WHEN_EXISTS,
                func = function (self, arg1)
                    ShowWhen (arg1)
                end,
            },
        } 
    },
    { 
        text = "Remove", 
        notCheckable = true,
        --arg1 = i,
        func = function (self) 
            curFrame:Hide()
            c_tblTrack[GetSpecialization()][curFrame.name] = nil
            c_tblTrackPos[GetSpecialization()][curFrame.name] = nil
            
            for k,v in pairs(MooUI_tblTrackPerSpec[GetSpecialization()]) do
                if v==curFrame then
                    MooUI_tblTrackPerSpec[GetSpecialization()][k] = nil
                    v.isActive = false
                end
            end
        end 
    },
    {
        text = "Cancel",
        notCheckable = true,
    },
}
local cmenuTrack = CreateFrame("Frame", "menu", UIParent, "UIDropDownMenuTemplate")
-- Example from http://www.wowwiki.com/API_EasyMenu
-- Details about the table from http://www.wowwiki.com/UI_Object_UIDropDownMenu#The_info_table

g_tblContextMenuTrack = tblContextMenuTrack -- I need it on global :/

------------------------------------------------------------------

function NewTrackingIcon (w, h, spell, target, icon, displayText, x, y, show)
    -- Returns a tracking icon with stuff in it!
    -- For ease of use, the follow sub-widgets' functions have been wrapped into the frame:
    -- SetTexture, SetCD, Text
    
    -- Create the frame
    local frm = CreateFrame ("Frame", GenerateFrameName("frm"..spell), UIParent)
    frm:SetSize (w, h)
    frm:SetPoint("CENTER", nil, "CENTER")
    frm:SetBackdrop ({
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        tile = true,
        tileSize = 32,
        edgeSize = 4,
    })
    frm:Raise ()
    frm.name = spell
    frm.target = target
    frm.showWhen = show or SHOW_ALWAYS
    if displayText == 'Value' then
        frm.Update = function (self)
            local val = select(15, _G.GetAuraData(self.target, self.name))
            self:SetText (_G.ShortNumber(val))
        end
    elseif displayText == 'Stacks' then
        frm.Update = function (self)
            local count = select(4, _G.GetAuraData(self.target, self.name))
            self:SetText (count)
        end
    elseif displayText == 'Time left' then
        frm.Update = function (self)
            local expires = select(7, _G.GetAuraData(self.target, self.name))
            self:SetText (_G.format('%.1f',expires-GetTime()))
        end
    end

    -- Create the texture
    frm.tex = frm:CreateTexture ('Texture', "HIGH")
    frm.tex:SetPoint ('BOTTOMLEFT', frm, 'BOTTOMLEFT', 0,0)
    frm.tex:SetSize (w,h)
    frm.tex:SetTexture (icon)

    -- Create the cooldown
    frm.cd = CreateFrame ('Cooldown', nil, frm)
    frm.cd:SetPoint ('BOTTOMLEFT', frm, 'BOTTOMLEFT', 1,1)
    frm.cd:SetSize (w-2,h-2)   
    
    -- Create the text    
    frm.stTxt = CreateFrame ('MessageFrame', nil, frm)
    frm.stTxt:SetSize(w-2,h/3.5+2)
    frm.stTxt:SetPoint ('BOTTOM', frm, 'BOTTOM', 0, 1)
    frm.stTxt:SetInsertMode ('BOTTOM')
    frm.stTxt:SetFading (false)
    frm.stTxt:SetTextColor (1.0,1.0,1.0,1.0)
    frm.stTxt:SetFont ('Fonts\\ARIALN.TTF', h/3.5, "")
    frm.stTxt:SetJustifyV('BOTTOM')
    frm.stTxt:SetFrameLevel(frm:GetFrameLevel()+1)
    frm.stTxt:SetShadowColor (0.0, 0.0, 0.0, 1.0)
    frm.stTxt:SetShadowOffset (-1, -1)
    frm.stTxt:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
        --edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        --edgeSize = 4,
    })

   
    -- Define wrapper methods
    frm.SetTexture = function (self, texture)
        self.tex:SetTexture (texture)
    end
    
    frm.SetCD = function (self, start, duration)
        self.cd:SetCooldown (start, duration)
    end

    frm.SetText = function (self, str)
        frm.stTxt:Clear ()
        if str=='' then
            frm.stTxt:Hide ()
        else
            frm.stTxt:Show ()
            frm.stTxt:AddMessage (str)
        end
    end

    -- Enable moving
    frm:SetMovable (true)
    frm:SetUserPlaced (true)
    frm:SetClampedToScreen (true)
    frm:SetScript("OnMouseDown", function (self, button, ...)
        if button=="RightButton" then
            curFrame = self
            for i,v in ipairs(tblContextMenuTrack[2].menuList) do
                v.checked = (v.arg1 == self.showWhen)
            end
            EasyMenu (tblContextMenuTrack, cmenuTrack, "cursor", 0 , 20, "MENU");
        elseif button=="LeftButton" then
            self:StartMoving ()
        end
    end)
    frm:SetScript("OnMouseUp", function (self, button, ...)
        if button=="LeftButton" then
            self:StopMovingOrSizing ()
            local point, relativeTo, relativePoint, xOffset, yOffset = self:GetPoint(1)
            xOffset = xOffset - xOffset%4
            yOffset = yOffset - yOffset%4
            self:SetPoint (point, relativeTo, relativePoint, xOffset, yOffset)
            
            if c_tblTrackPos[GetSpecialization()] == nil then
                c_tblTrackPos[GetSpecialization()] = {}
            end
            c_tblTrackPos[GetSpecialization()][self.name] = {point, relativeTo, relativePoint, xOffset, yOffset}
        end
    end)

    -- Some methods that the timers are going to need:
    frm.Dark = function (self)
        self:SetCD (GetTime()-1,1)
        self.cd:SetReverse(true)
        if self.SetText then
            self:SetText ('')
        end
        if self.SetBarValue then
            self:SetBarValue (select(1,self.stBar:GetMinMaxValues()))
        end
    end
    
    frm:SetScript("OnUpdate", function (self,event,...)
        local name,_,_,_,_, duration, expires = UnitDebuff(self.target, self.name:trim())
        if name==nil then
            name,_,_,_,_, duration, expires = UnitAura(self.target, self.name:trim())
        end
        
        if name~=nil then
            if self.Update then 
                self:Update() 
            end
            self:SetCD (expires-duration, duration)
        else
            self:Dark ()
        end    
    end)
    
    -- Add a tooltip
    frm:SetScript("OnEnter", function (self,motion,...)
        GameTooltip:SetOwner (self, "ANCHOR_BOTTOMRIGHT")
        local name = GetAuraData(self.target, self.name:trim())
        if name==nil then
            GameTooltip:SetText (self.name..' - '..self.target)
        else
            GameTooltip:SetUnitAura (self.target, self.name:trim())
        end
        
        GameTooltip:Show ()
    end)
    
    frm:SetScript("OnLeave", function (self,motion,...)
        GameTooltip:Hide ()
    end)
    
    frm:RegisterUnitEvent ("UNIT_AURA", frm.target)
    frm:RegisterEvent ("PLAYER_REGEN_ENABLED")
    frm:RegisterEvent ("PLAYER_REGEN_DISABLED")
    frm:SetScript ("OnEvent", function (self, event)
        if not self.isActive then return end
        
        if event=="UNIT_AURA" then
            if self.showWhen == SHOW_WHEN_EXISTS then
                local name = _G.GetAuraData(self.target, self.name:trim())
                if name then
                    self:Show ()
                else
                    self:Hide()
                end
            end     
        elseif event=="PLAYER_REGEN_ENABLED" then
            if self.showWhen == SHOW_IN_COMBAT then
                self:Hide()
            end
        elseif event=="PLAYER_REGEN_DISABLED" then
            if self.showWhen == SHOW_IN_COMBAT then
                self:Show()
            end
        end
    end)
            
    -- Init stuff
    frm:Dark()
    frm:Hide()
    frm:EnableMouse (not c_lock)    -- Has to happen here - apparently setting mouse events enables the mouse -.o
    
    return frm
end

------------------------------------------------------------------

function NewTrackingBar (w, h, spell, target, icon, displayText, barType, color, orientation, show)
    -- Creates and returns a tracking bar
    -- orientation is HORIZONTAL or VERTICAL
    
    local f = CreateFrame ("StatusBar", nil, UIParent)
    f:SetSize (w, h)
    f:SetMinMaxValues (0,1)
    f:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
    f:SetOrientation (orientation)
    if color then
        f:SetStatusBarColor (unpack(color))
    end
    
    f.spellId = spellId
    f.type = barType
    
    f.name = spell
    f.target = target
    f.showWhen = show or SHOW_ALWAYS
    
    f.txt = f:CreateFontString(nil, "HIGH", "GameFontNormal")
    f.txt:SetAllPoints (f)
    f.txt:SetTextColor (1.0,1.0,1.0)
    f.txt:SetJustifyH ("CENTER")
    f.txt:SetTextHeight (11)
    
    f.tex = f:CreateTexture ("Texture", "HIGH")
    f.tex:SetSize (ICON_SIZE, ICON_SIZE)
    f.tex:SetPoint ("RIGHT", f, "LEFT")
    
    local name,_,icon = GetSpellInfo (spellId)
    f.tex:SetTexture (icon)
    f.name = displayName or name
    
    function f:Update (tblAuras)
        local valBar, valText, valMax
        if tblAuras[self.spellId] then
            if self.type == 'time' then
                valText = format ('%.1f', tblAuras[self.spellId]['expires'] - GetTime())
            elseif self.type == 'value' then
                valText = ShortNumber(tblAuras[self.spellId]['value2'])
            elseif self.type == 'stacks' then
                valText = tblAuras[self.spellId]['count']
            end
            valBar = tblAuras[self.spellId]['expires'] - GetTime()
            valMax = tblAuras[self.spellId]['duration']
        else
            valBar, valText, valMax = 0,0,1
        end
        
        self:SetMinMaxValues (0, valMax)
        self:SetValue (valBar)
        self.txt:SetText (valText)
    end
    
    function f:SetTooltipText (txt)
        self:SetScript("OnEnter", function (self,motion,...)
            GameTooltip:SetOwner (self, "ANCHOR_RIGHT")
            GameTooltip:SetText (txt)
            GameTooltip:Show ()
        end)
        
        self:SetScript("OnLeave", function (self,motion,...)
            GameTooltip:Hide ()
        end)
    end
    
    --return f
        
        ----------- ------- ----- -
    
    -- Create the frame
    local frm = CreateFrame ("Frame", GenerateFrameName("frm"..spell), UIParent)
    frm:SetSize (w, h)
    frm:SetPoint("CENTER", nil, "CENTER")
    frm:SetBackdrop ({
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        tile = true,
        tileSize = 32,
        edgeSize = 4,
    })
    frm:Raise ()
    frm.name = spell
    frm.target = target
    frm.showWhen = show or SHOW_ALWAYS
    if displayText == 'Value' then
        frm.Update = function (self)
            local val = select(15, _G.GetAuraData(self.target, self.name))
            self:SetText (_G.ShortNumber(val))
        end
    elseif displayText == 'Stacks' then
        frm.Update = function (self)
            local count = select(4, _G.GetAuraData(self.target, self.name))
            self:SetText (count)
        end
    elseif displayText == 'Time left' then
        frm.Update = function (self)
            local expires = select(7, _G.GetAuraData(self.target, self.name))
            self:SetText (_G.format('%.1f',expires-GetTime()))
        end
    end

    -- Create the texture
    frm.tex = frm:CreateTexture ('Texture', "HIGH")
    frm.tex:SetPoint ('BOTTOMLEFT', frm, 'BOTTOMLEFT', 0,0)
    frm.tex:SetSize (w,h)
    frm.tex:SetTexture (icon)

    -- Create the cooldown
    frm.cd = CreateFrame ('Cooldown', nil, frm)
    frm.cd:SetPoint ('BOTTOMLEFT', frm, 'BOTTOMLEFT', 1,1)
    frm.cd:SetSize (w-2,h-2)   
    
    -- Create the text    
    frm.stTxt = CreateFrame ('MessageFrame', nil, frm)
    frm.stTxt:SetSize(w-2,h/3.5+2)
    frm.stTxt:SetPoint ('BOTTOM', frm, 'BOTTOM', 0, -1)
    frm.stTxt:SetInsertMode ('BOTTOM')
    frm.stTxt:SetFading (false)
    frm.stTxt:SetTextColor (1.0,1.0,1.0,1.0)
    frm.stTxt:SetFont ('Fonts\\ARIALN.TTF', h/3.5, "")
    frm.stTxt:SetJustifyV('BOTTOM')
    frm.stTxt:SetFrameLevel(frm:GetFrameLevel()+1)
    frm.stTxt:SetShadowColor (0.0, 0.0, 0.0, 1.0)
    frm.stTxt:SetShadowOffset (-1, -1)
    frm.stTxt:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        edgeSize = 4,
    })

   
    -- Define wrapper methods
    frm.SetTexture = function (self, texture)
        self.tex:SetTexture (texture)
    end
    
    frm.SetCD = function (self, start, duration)
        self.cd:SetCooldown (start, duration)
    end

    frm.SetText = function (self, str)
        frm.stTxt:Clear ()
        if str=='' then
            frm.stTxt:Hide ()
        else
            frm.stTxt:Show ()
            frm.stTxt:AddMessage (str)
        end
    end

    -- Enable moving
    frm:SetMovable (true)
    frm:SetUserPlaced (true)
    frm:SetClampedToScreen (true)
    frm:SetScript("OnMouseDown", function (self, button, ...)
        if button=="RightButton" then
            curFrame = self
            for i,v in ipairs(tblContextMenuTrack[2].menuList) do
                v.checked = (v.arg1 == self.showWhen)
            end
            EasyMenu (tblContextMenuTrack, cmenuTrack, "cursor", 0 , 20, "MENU");
        elseif button=="LeftButton" then
            self:StartMoving ()
        end
    end)
    frm:SetScript("OnMouseUp", function (self, button, ...)
        if button=="LeftButton" then
            self:StopMovingOrSizing ()
            local point, relativeTo, relativePoint, xOffset, yOffset = self:GetPoint(1)
            xOffset = xOffset - xOffset%4
            yOffset = yOffset - yOffset%4
            self:SetPoint (point, relativeTo, relativePoint, xOffset, yOffset)
            
            if c_tblTrackPos[GetSpecialization()] == nil then
                c_tblTrackPos[GetSpecialization()] = {}
            end
            c_tblTrackPos[GetSpecialization()][self.name] = {point, relativeTo, relativePoint, xOffset, yOffset}
        end
    end)

    -- Some methods that the timers are going to need:
    frm.Dark = function (self)
        self:SetCD (GetTime()-1,1)
        self.cd:SetReverse(true)
        if self.SetText then
            self:SetText ('')
        end
        if self.SetBarValue then
            self:SetBarValue (select(1,self.stBar:GetMinMaxValues()))
        end
    end
    
    frm:SetScript("OnUpdate", function (self,event,...)
        local name,_,_,_,_, duration, expires = UnitDebuff(self.target, self.name:trim())
        if name==nil then
            name,_,_,_,_, duration, expires = UnitAura(self.target, self.name:trim())
        end
        
        if name~=nil then
            if self.Update then 
                self:Update() 
            end
            self:SetCD (expires-duration, duration)
        else
            self:Dark ()
        end    
    end)
    
    -- Add a tooltip
    frm:SetScript("OnEnter", function (self,motion,...)
        GameTooltip:SetOwner (self, "ANCHOR_BOTTOMRIGHT")
        local name = GetAuraData(self.target, self.name:trim())
        if name==nil then
            GameTooltip:SetText (self.name..' - '..self.target)
        else
            GameTooltip:SetUnitAura (self.target, self.name:trim())
        end
        
        GameTooltip:Show ()
    end)
    
    frm:SetScript("OnLeave", function (self,motion,...)
        GameTooltip:Hide ()
    end)
    
    frm:RegisterEvent ("UNIT_AURA")
    frm:RegisterEvent ("PLAYER_REGEN_ENABLED")
    frm:RegisterEvent ("PLAYER_REGEN_DISABLED")
    frm:SetScript ("OnEvent", function (self, event)
        if not self.isActive then return end
        
        if event=="UNIT_AURA" then
            if self.showWhen == SHOW_WHEN_EXISTS then
                local name = _G.GetAuraData(self.target, self.name:trim())
                if name then
                    self:Show ()
                else
                    self:Hide()
                end
            end     
        elseif event=="PLAYER_REGEN_ENABLED" then
            if self.showWhen == SHOW_IN_COMBAT then
                self:Hide()
            end
        elseif event=="PLAYER_REGEN_DISABLED" then
            if self.showWhen == SHOW_IN_COMBAT then
                self:Show()
            end
        end
    end)
            
    -- Init stuff
    frm:Dark()
    frm:Hide()
    frm:EnableMouse (not c_lock)    -- Has to happen here - apparently setting mouse events enables the mouse -.o
    
    return frm
end

------------------------------------------------------------------

local function NewLabeledTextbox (parent, text, h, y)
    local lbl = parent:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    lbl:SetSize (parent:GetWidth()*0.82, h)
    lbl:SetPoint ("TOP", parent, "TOP", 0, y)
    lbl:SetText(text)

    local txt = CreateFrame("EditBox", nil, parent)
    txt:SetPoint ("TOP", parent, "TOP", 0, y-h-1)
    txt:SetSize (parent:GetWidth()*0.82, h)
    txt:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
        tile = true,
        tileSize = 32,
        edgeSize = 6,
    })
    txt:SetTextColor (1.0,1.0,1.0,1.0)
    txt:SetFont ('Fonts\\ARIALN.TTF', 14, "")
    txt:SetAutoFocus (false)
    txt:SetTextInsets (2, 2, 0, 0)
    txt:SetScript ("OnEscapePressed", function(self)
        self:ClearFocus ()
    end)
    
    return lbl, txt
end

------------------------------------------------------------------

local function NewButton (parent, text, h, y)
    local btn=CreateFrame ("Button", nil, parent, "UIPanelButtonTemplate")
    btn:SetSize (parent:GetWidth()*0.83, h)
    btn:SetPoint ("TOP", parent, "TOP", 0, y)
    btn:SetText(text)
    
    return btn
end

------------------------------------------------------------------

local function NewCheckBox (parent, text, h, y)
    local lbl = parent:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    lbl:SetSize (parent:GetWidth()*0.74, h)
    lbl:SetPoint ("TOP", parent, "TOP", 12, y)
    lbl:SetText(' '..text)
    lbl:SetJustifyH ("LEFT")

    local chk = CreateFrame ("CheckButton", nil, parent, "UICheckButtonTemplate")
    chk:SetSize (24,24)
    chk:SetPoint ("RIGHT", lbl, "LEFT", 0, -1)

    return lbl, chk
end

------------------------------------------------------------------

 function CreateTrackDialog ()
    local frm = CreateFrame ("Frame", "frmNewTrack", UIParent)
    frm:SetPoint ("CENTER", UIParent, "CENTER")
    frm:SetWidth (180)
    frm:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
        edgeSize = 8,
    })
    
    local y = -11
    
    -- Spell Name
    frm.lblName, frm.txtName = NewLabeledTextbox (frm, 'Spell Name', 20, y)
    frm.txtName:SetScript("OnTextChanged", function()
        frm.txtIcon:SetText (frm.txtName:GetText())
    end )
    y=y-44
    
    -- Target
    frm.lblTarget, frm.txtTarget = NewLabeledTextbox (frm, 'Target', 20, y)
    y=y-41
    
    frm.btnMyself= NewButton (frm, "Myself", 22, y)
    frm.btnMyself:SetScript("OnClick", function()
        frm.txtTarget:SetText ('player')
    end)
    y=y-19

    frm.btnTarget = NewButton (frm, "My target", 22, y)
    frm.btnTarget:SetScript("OnClick", function()
        if UnitName('target') then
            frm.txtTarget:SetText (UnitName('target'))
        end
    end)
    y=y-39
    
    -- Icon selection
    frm.lblIcon, frm.txtIcon = NewLabeledTextbox (frm, 'Icon Name', 20, y)
    y=y-41
    
    frm.btnFindIcon = NewButton (frm, "Search...", 22, y)
    frm.btnFindIcon:SetScript("OnClick", function()
        frm.lstIcons = FindIcons (frm.txtIcon:GetText():trim())
        if #frm.lstIcons > 0 then
            frm.curIconNum = 1
            frm.lblFound:SetText (frm.curIconNum.." of "..#frm.lstIcons)
            frm.ShowIcon (1)
        else
            frm.lblFound:SetText ('No such icons :(')
        end
    end)
    y=y-28
    
    frm.icon = frm:CreateTexture ('Texture', "HIGH")
    frm.icon:SetPoint ('TOP', frm, 'TOP', 0, y)
    frm.icon:SetSize (48, 48)
    y=y-70
    
    frm.ShowIcon = function (i)
        -- The parameter is an index between 1 and #frm.lstIcons.
        -- Or, to make it easier to use modulu, if the paramter is 0 it is treated like #frm.lstIcons
        if i==0 then i=#frm.lstIcons end
        if i<1 or i>#frm.lstIcons then return end
        
        frm.curIconNum = i
        frm.icon:SetTexture (frm.lstIcons[i])
        frm.lblFound:SetText (i.." of "..#frm.lstIcons)
    end
    
    frm.lblFound = frm:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    frm.lblFound:SetSize (frm:GetWidth()*0.82, 20)
    frm.lblFound:SetPoint ("TOP", frm.icon, "BOTTOM")
    
    -- Next Icon button
    frm.btnNextIcon = CreateFrame ("Button", nil, frm)
    frm.btnNextIcon:SetSize (24,32)
    frm.btnNextIcon:SetPoint ("LEFT", frm.icon, "RIGHT")
    frm.btnNextIcon:SetNormalTexture ("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up")
    frm.btnNextIcon:SetPushedTexture ("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down")
    frm.btnNextIcon:SetHighlightTexture ("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")
    frm.btnNextIcon:SetScript ("OnClick", function()
        if #frm.lstIcons==0 then return end
        
        frm.ShowIcon ((frm.curIconNum+1) % #frm.lstIcons)
    end)
    
    -- Previous Icon button
    frm.btnPrevIcon = CreateFrame ("Button", nil, frm)
    frm.btnPrevIcon:SetSize (24,32)
    frm.btnPrevIcon:SetPoint ("RIGHT", frm.icon, "LEFT")
    frm.btnPrevIcon:SetNormalTexture ("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up")
    frm.btnPrevIcon:SetPushedTexture ("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down")
    frm.btnPrevIcon:SetHighlightTexture ("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")
    frm.btnPrevIcon:SetScript ("OnClick", function()
        if #frm.lstIcons==0 then return end
        
        frm.ShowIcon ((frm.curIconNum-1) % #frm.lstIcons)
    end)
    
    -- Dropdown menu for display text
    frm.lblDisplayText = frm:CreateFontString (nil, "BACKGROUND", "GameFontNormal")
    frm.lblDisplayText:SetSize (frm:GetWidth()*0.82, 20)
    frm.lblDisplayText:SetPoint ("TOP", frm, "TOP", 0, y)
    frm.lblDisplayText:SetText ("Display text:")
    y=y-21
    
    frm.lstDisplayTextOptions = {'Value', 'Stacks', 'Time left', 'None'}
    frm.menuDisplayText = CreateFrame("Button", "MooUI_Track_menuDisplayText", frm, "UIDropDownMenuTemplate")
    frm.menuDisplayText:SetPoint("TOP", frm, "TOP", 0, y)
    UIDropDownMenu_Initialize (frm.menuDisplayText, function (self, level)
        local info = UIDropDownMenu_CreateInfo()
        for k,v in pairs(frm.lstDisplayTextOptions) do
            info = UIDropDownMenu_CreateInfo()
            info.text = v
            info.value = v
            info.func = function (self)
                UIDropDownMenu_SetSelectedID(frm.menuDisplayText, self:GetID())
            end
            UIDropDownMenu_AddButton(info, level)
        end
    end)
    UIDropDownMenu_SetWidth (frm.menuDisplayText, 100);
    UIDropDownMenu_SetButtonWidth (frm.menuDisplayText, 124)
    UIDropDownMenu_SetSelectedID (frm.menuDisplayText, 1)
    UIDropDownMenu_JustifyText (frm.menuDisplayText, "LEFT")
    y=y-56

    -- Size slider
    frm.sldSize = CreateFrame("Slider", frm:GetName()..'_sldSize', frm, "OptionsSliderTemplate")
    frm.sldSize:SetSize (frm:GetWidth()*0.82, 24)
    frm.sldSize:SetPoint ("TOP", frm, "TOP", 0, y)
    _G[frm.sldSize:GetName() .. 'Low']:SetText('16');
    _G[frm.sldSize:GetName() .. 'High']:SetText('64');
    frm.sldSize:SetMinMaxValues (16, 64)
    frm.sldSize:SetValue (40)
    frm.sldSize:SetValueStep (8)
    y=y-36
    
    frm.lblSize = frm:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    frm.lblSize:SetSize (frm:GetWidth()*0.82, 22)
    frm.lblSize:SetPoint ("BOTTOM", frm.sldSize, "TOP", 0, -2)
    frm.lblSize:SetText('Size')

    -- Dropdown menu for when to show
    frm.lblShowWhen = frm:CreateFontString (nil, "BACKGROUND", "GameFontNormal")
    frm.lblShowWhen:SetSize (frm:GetWidth()*0.82, 20)
    frm.lblShowWhen:SetPoint ("TOP", frm, "TOP", 0, y)
    frm.lblShowWhen:SetText ("Show icon:")
    y=y-19
    
    frm.lstShowWhenOptions = {"Always", "In combat", "When the aura exists"}
    frm.menuShowWhen = CreateFrame("Button", "MooUI_Track_menuShowWhen", frm, "UIDropDownMenuTemplate")
    frm.menuShowWhen:SetPoint("TOP", frm, "TOP", 0, y)
    UIDropDownMenu_Initialize (frm.menuShowWhen, function (self, level)
        local info = UIDropDownMenu_CreateInfo()
        for k,v in pairs(frm.lstShowWhenOptions) do
            info = UIDropDownMenu_CreateInfo()
            info.text = v
            info.value = v
            info.func = function (self)
                UIDropDownMenu_SetSelectedID(frm.menuShowWhen, self:GetID())
            end
            UIDropDownMenu_AddButton(info, level)
        end
    end)
    UIDropDownMenu_SetWidth (frm.menuShowWhen, 100);
    UIDropDownMenu_SetButtonWidth (frm.menuShowWhen, 124)
    UIDropDownMenu_SetSelectedID (frm.menuShowWhen, 1)
    UIDropDownMenu_JustifyText (frm.menuShowWhen, "LEFT")
    y=y-40

    -- And finally - Track button
    frm.btnTrack = NewButton (frm, "Track!", 22, -468)
    frm.btnTrack:SetScript("OnClick", function (self)
        local frm = self:GetParent()
        local spec = GetSpecialization()
        if c_tblTrack[spec] and c_tblTrack[spec][frm.txtName:GetText()] then
            print ("You're already tracking this ability for this spec!")
            return
        end
        
        local params = {
            frm.sldSize:GetValue(), frm.sldSize:GetValue(),
            frm.txtName:GetText(),
            frm.txtTarget:GetText(),
            frm.lstIcons[frm.curIconNum],
            UIDropDownMenu_GetText (MooUI_Track_menuDisplayText),
            nil,
            nil,
            UIDropDownMenu_GetSelectedID (MooUI_Track_menuShowWhen)
        }
        local f = NewTrackingIcon (unpack(params))
        f.isActive = true
        
        c_tblTrack[spec] = c_tblTrack[spec] or {}
        c_tblTrack[spec][frm.txtName:GetText()] = params
        
        MooUI_tblTrackPerSpec[spec] = MooUI_tblTrackPerSpec[spec] or {}
        table.insert (MooUI_tblTrackPerSpec[spec], f)
                
        f:GetScript("OnMouseUp")(f, "LeftButton")
        
        f:Show()
        f:SetPoint ("CENTER", UIParent, "CENTER")
        frm:Hide()        
    end )
    y=y-36
    
    -- Make movable
    frm:SetMovable (true)
    frm:RegisterForDrag ("LeftButton")
    frm:SetUserPlaced (true)
    frm:SetClampedToScreen (true)
    frm:EnableMouse (true)
    frm:SetScript("OnDragStart", function(self, button)
        self:StartMoving ()
    end)
    frm:SetScript("OnDragStop", function(self)
        self:StopMovingOrSizing ()
    end)
    
    -- x button
    NewButtonClose(frm,-2,-2,24,24)
    
    -- Tabs!
    local tmp = {frm.txtName, frm.txtTarget, frm.txtIcon}
    for i,v in ipairs(tmp) do
        local n = i+1
        if n==#tmp+1 then n=1 end
        v.next = tmp[n]
        v:SetScript ("OnTabPressed", function (self)
            self.next:SetFocus()
        end)
    end
    
    frm:SetHeight(-y)
    
    return frm
end

frmNewTrack = CreateTrackDialog()
frmNewTrack:Hide()

------------------------------------------------------------------

function LockAll (b)
    if MooUI_tblTrack then
        for i,v in ipairs(MooUI_tblTrack)  do
            v:EnableMouse (b)
        end
    end
end

------------------------------------------------------------------

SLASH_TRACK1 = "/track"
SlashCmdList["TRACK"] = function (txt)
    if #txt==0 then
        frmNewTrack.txtName:SetText ('')
        frmNewTrack.txtTarget:SetText ('')
        frmNewTrack.txtIcon:SetText ('')
        frmNewTrack.icon:SetTexture ("Interface\\DialogFrame\\UI-DialogBox-Background")
        frmNewTrack:Show()
        frmNewTrack.txtName:SetFocus()
    elseif txt=='reset spec' then
        c_tblTrack[GetSpecialization()] = {}
        c_tblTrackPos[GetSpecialization()] = {}
        ReloadUI ()
    elseif txt=='reset all' then
        c_tblTrack = {}
        c_tblTrackPos = {}
        ReloadUI ()
    elseif txt=='show' then
        if MooUI_tblTrack then
            for i,v in ipairs(MooUI_tblTrack)  do
                v:Show()
            end
        end
    elseif txt=='lock' then
        c_lock = true
        LockAll (not c_lock)
    elseif txt=='unlock' then
        c_lock = false
        LockAll (not c_lock)
    end
end

------------------------------------------------------------------

local function HideCurrentTracking ()
    if not MooUI_tblTrack then return end
    for i,v in ipairs(MooUI_tblTrack)  do
        v:Hide()
        v.isActive = false
    end
end

local function ShowCurrentTracking ()
    if not MooUI_tblTrack then return end
    for i,v in ipairs(MooUI_tblTrack)  do
        v.isActive = true
        if v.showWhen == SHOW_ALWAYS then
            v:Show()
        end
    end
end

------------------------------------------------------------------

local frmShowHide = CreateFrame ("Frame", nil, UIParent)
frmShowHide:RegisterEvent ("PET_BATTLE_OPENING_START")
frmShowHide:RegisterEvent ("PET_BATTLE_CLOSE")
frmShowHide:RegisterUnitEvent ("UNIT_ENTERED_VEHICLE", 'player')
frmShowHide:RegisterUnitEvent ("UNIT_EXITED_VEHICLE", 'player')
frmShowHide:RegisterEvent ("CINEMATIC_START")
frmShowHide:RegisterEvent ("CINEMATIC_STOP")
frmShowHide:RegisterEvent ("CINEMATIC_STOP")
frmShowHide:RegisterEvent ("PLAYER_LOGIN")
frmShowHide:RegisterEvent ("PLAYER_SPECIALIZATION_CHANGED")
frmShowHide:SetScript ("OnEvent", function (self, event)
    if event=='PET_BATTLE_OPENING_START'  or  event=='CINEMATIC_START'  or  event=='UNIT_ENTERED_VEHICLE'  then
        HideCurrentTracking()
    elseif event=='PET_BATTLE_CLOSE'  or  event=='CINEMATIC_STOP'  or  event=='UNIT_EXITED_VEHICLE'  then
        ShowCurrentTracking()
    elseif event=='PLAYER_LOGIN' then
        -- Load all tracking stuff, prepare the interesting tables...
        if c_tblTrack==nil      then c_tblTrack = {}    end
        if c_tblTrackPos==nil   then c_tblTrackPos = {} end
        
        for i=0,NUM_SPECS do
            MooUI_tblTrackPerSpec[i] = {}
        end
        for spec,tbl in pairs(c_tblTrack) do
            for k,v in pairs(tbl) do
                local f = NewTrackingIcon (unpack(v))
                f:SetPoint (unpack(c_tblTrackPos[spec][k]))
                table.insert (MooUI_tblTrackPerSpec[spec], f)
            end
        end

        -- And now show the relevant stuff
        MooUI_tblTrack = MooUI_tblTrackPerSpec[GetSpecialization()]
        if MooUI_tblTrack==nil then
            MooUI_tblTrack = {}
        end
        ShowCurrentTracking()
    elseif event=='PLAYER_SPECIALIZATION_CHANGED' then
        HideCurrentTracking()
        MooUI_tblTrack = MooUI_tblTrackPerSpec[GetSpecialization()]
        ShowCurrentTracking()
    end
end)


------------------------------------------------------------------
